//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//-------------------------------------------------------------------------------
// Copyright (C) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//-------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.ComponentModel.Design;
using System.Globalization;
using System.Web.UI.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using DreamDriven.SoftwareFactory.Components.Properties;

namespace DreamDriven.Components.DataSource.Design
{
	/// <summary>
	/// Provides design-time support in a design host for the <see cref="DomainObjectDataSource"/> control.
	/// </summary>
    public class DomainObjectDataSourceDesigner : DataSourceDesigner
    {
        private readonly string[] _viewNames = { Resources.DefaultViewName };

		/// <summary>
		/// Indicates whether the <see cref="System.Web.UI.Design.DataSourceDesigner.RefreshSchema(System.Boolean)"/> method can be called.
		/// </summary>
        public override bool CanRefreshSchema
        {
            get
            {
                if (!string.IsNullOrEmpty(DataObjectTypeName))
                {
                    return TypeServiceAvailable;
                }
                return false;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the <see cref="M:System.Web.UI.Design.DataSourceDesigner.Configure"/> method can be called.
        /// </summary>
        /// <value></value>
        /// <returns>true if <see cref="M:System.Web.UI.Design.DataSourceDesigner.Configure"/> can be called; otherwise, false. The default is false.</returns>
        public override bool CanConfigure
        {
            get { return TypeServiceAvailable; }
        }

        /// <summary>
        /// Gets or sets the name of the data object type.
        /// </summary>
        /// <value>The name of the data object type.</value>
        internal string DataObjectTypeName
        {
            get
            {
                return DataSourceControl.DataObjectTypeName;
            }
            set
            {
                if (value != DataObjectTypeName)
                {
                    DataSourceControl.DataObjectTypeName = value;
                    UpdateDesignTimeHtml();
                    if (CanRefreshSchema)
                    {
                        RefreshSchema(true);
                    }
                    else
                    {
                        OnDataSourceChanged(EventArgs.Empty);
                    }
                }
            }
        }

        private bool TypeServiceAvailable
        {
            get
            {
                IServiceProvider serviceProvider = Component.Site;
                if (serviceProvider != null)
                {
                    var typeResolutionService = (ITypeResolutionService)serviceProvider.GetService(typeof(ITypeResolutionService));
                    var typeDiscoveryService = (ITypeDiscoveryService)serviceProvider.GetService(typeof(ITypeDiscoveryService));
                    return (typeResolutionService != null || typeDiscoveryService != null);
                }
                return false;
            }
        }

        /// <summary>
        /// Gets the data source control.
        /// </summary>
        /// <value>The data source control.</value>
        internal DomainObjectDataSource DataSourceControl
        {
            get { return (DomainObjectDataSource)Component; }
        }

        /// <summary>
        /// Gets or sets the data source schema.
        /// </summary>
        /// <value>The data source schema.</value>
        internal IDataSourceViewSchema DataSourceSchema
        {
            get { return DesignerState["DataSourceSchema"] as IDataSourceViewSchema; }
            set { DesignerState["DataSourceSchema"] = value; }
        }

        /// <summary>
        /// Retrieves a <see cref="DesignerDataSourceView"/> object that is identified by the view name.
        /// </summary>
        /// <param name="viewName">The name of the view.</param>
        /// <returns>
        /// The <see cref="DesignerDataSourceView"/> identified by the view name.
        /// </returns>
        /// <exception cref="ArgumentException">The view name is equal to the default view name.</exception>
        public override DesignerDataSourceView GetView(string viewName)
        {
            if (!string.Equals(viewName, Resources.DefaultViewName, StringComparison.OrdinalIgnoreCase) && !String.IsNullOrEmpty(viewName))
            {
                string message = String.Format(CultureInfo.CurrentCulture, Resources.InvalidViewName, "viewName");
                throw new ArgumentException(message);
            }

		    return new DomainObjectDataSourceDesignerView(this, viewName);
        }

        /// <summary>
        /// Launches the data source configuration utility in the design host.
        /// </summary>
        /// <exception cref="T:System.NotSupportedException">An attempt was made to invoke this method in the base class.</exception>
        public override void Configure()
        {
            InvokeTransactedChange(Component, ConfigureDataSourceChangeCallback, null, "Configure Data Source");
        }

		/// <summary>
		/// Indicates whether to allow events when refreshing the schema.
		/// </summary>
		/// <param name="preferSilent"><see langword=" true"/> to allow events when refreshing the schema; 
		/// <see langword=" false"/> to disable the DataSourceChanged and SchemaRefreshed events when refreshing the schema.</param>
        public override void RefreshSchema(bool preferSilent)
        {
            try
            {
                base.SuppressDataSourceEvents();
                Cursor cursor = Cursor.Current;
                try
                {
                    Cursor.Current = Cursors.WaitCursor;
                    Type type = GetType(Component.Site, DataObjectTypeName, preferSilent);

                    if (type == null)
                        return;
                    
                    IDataSourceViewSchema[] schemas = new TypeSchema(type).GetViews();
                    if ((schemas == null) || (schemas.Length == 0))
                    {
                        DataSourceSchema = null;
                        return;
                    }

                    IDataSourceViewSchema newSchema = schemas[0];
                    IDataSourceViewSchema oldSchema = DataSourceSchema;
                    if (!ViewSchemasEquivalent(oldSchema, newSchema))
                    {
                        DataSourceSchema = newSchema;
                        OnSchemaRefreshed(EventArgs.Empty);
                    }
                }
                catch (Exception ex)
                {
                    string message = String.Format(CultureInfo.CurrentCulture, Resources.CannotRefreshSchema, ex.Message);
                    DisplayErrorMessage(DataSourceControl.Site, message);
                }
                finally
                {
                    Cursor.Current = cursor;
                }
            }
            finally
            {
                base.ResumeDataSourceEvents();
            }
        }

		/// <summary>
		/// Retrieves the view names that are available in this data source.
		/// </summary>
		/// <returns>An array of <see langword="string"/>.</returns>
        public override string[] GetViewNames()
        {
            return _viewNames;
        }

        internal static Type GetType(IServiceProvider serviceProvider, string typeName, bool preferSilent)
        {
            var typeResolutionService = GetService<ITypeResolutionService>(serviceProvider);
            if (typeResolutionService == null)
                return null;

            try
            {
                return typeResolutionService.GetType(typeName, true, true);
            }
            catch (Exception ex)
            {
                if (!preferSilent)
                {
                    string message = String.Format(CultureInfo.CurrentCulture, Resources.CannotGetType, ex.Message);
                    DisplayErrorMessage(serviceProvider, message);
                }
                return null;
            }
        }

        private static T GetService<T>(IServiceProvider serviceProvider)
            where T : class
        {
            if (serviceProvider == null)
            {
                return null;
            }

            return (T)serviceProvider.GetService(typeof(T));
        }

        private bool ConfigureDataSourceChangeCallback(object context)
        {
            try
            {
                SuppressDataSourceEvents();
                IUIService uiService = GetService<IUIService>(Component.Site);
                if (uiService == null)
                {
                    return false;
                }

                var form = new DomainObjectDataSourceWizardForm(this);
                DialogResult result = uiService.ShowDialog(form);
                if (result == DialogResult.OK)
                {
                    OnDataSourceChanged(EventArgs.Empty);
                    return true;
                }
            }
            finally
            {
                ResumeDataSourceEvents();
            }
            return false;
        }

        private static void DisplayErrorMessage(IServiceProvider serviceProvider, string message)
        {
            var service = GetService<IUIService>(serviceProvider);
            if (service != null)
            {
                service.ShowError(message);
            }
        }
    }
}
